rm(list = ls())
library(data.table)
library(dplyr)
library(tidyr)
library(survival)
# path = "/Users/jianchen/Library/CloudStorage/Dropbox/Yang/UNC/covid_optimal_booster/Results/output_age65+_3"
# load("../../prediction/for_bios/entry_time_age65+.RData")

load("../entry_time_age65+_inf.RData")
entry.time.inf = data.frame(id = 1:length(VacTime), entry.time = VacTime)
load("../entry_time_age65+_svr.RData")
entry.time.svr = data.frame(id = 1:length(VacTime), entry.time = VacTime)
filename = dir("../output_age65+_3")

get_plot_data = function(pred_svr, entry.time){
  dat = pred_svr %>%
    select(id, infection.time.1, infection.time.2, infection.time.3, 
           infection.time.4, infection.time.5) %>%
    gather(infection.time.1, infection.time.2, infection.time.3, infection.time.4, 
           infection.time.5, key = "infection.num", value = "infection.time") %>%
    filter(infection.time < 999) %>%
    arrange(id)
  index = which(dat$infection.time >= tau & duplicated(dat$id, fromLast = F))
  dat = dat[-index,]
  
  ind_dup1 = duplicated(dat$id, fromLast = T)
  ind_dup2 = duplicated(dat$id, fromLast = F)
  ind_dup = ind_dup1 | ind_dup2
  dat_dup = dat[ind_dup,]
  dat_uni = dat[!ind_dup,]
  
  ##### Duplicated
  surv_dup = dat_dup %>%
    mutate(stop = infection.time, status  = 1) %>%
    select(id, stop, status)
  tmp = dat_dup[!duplicated(dat_dup$id),] %>%
    mutate(stop = tau, status = 0) %>%
    select(id, stop, status)
  surv_dup = rbind(surv_dup,tmp) %>%
    arrange(id)
  rownames(surv_dup) = NULL
  
  tmp1 = entry.time %>%
    filter(id %in% surv_dup$id) %>%
    rename(start = entry.time)
  tmp2 = surv_dup[duplicated(surv_dup$id, fromLast = T),] %>%
    select(id, stop) %>%
    rename(start = stop)
  tmp = rbind(tmp1, tmp2) %>% arrange(id)
  surv_dup = surv_dup %>%
    mutate(start = tmp$start) %>%
    select(id, start, stop, status)
  
  #### Unique
  ind10 = which(dat_uni$infection.time < tau)
  ind00 = which(dat_uni$infection.time >= tau)
  tmp10 = dat_uni[ind10,]
  tmp00 = dat_uni[ind00,]
  surv00 = data.frame(id = tmp00$id, 
                      start = entry.time$entry.time[which(entry.time$id %in% tmp00$id)],
                      stop = tau,
                      status = 0)
  surv10 = data.frame(id = tmp10$id, 
                      start = entry.time$entry.time[which(entry.time$id %in% tmp10$id)],
                      stop = tmp10$infection.time,
                      status = 1)
  surv_tmp = data.frame(id = tmp10$id, 
                        start = tmp10$infection.time,
                        stop = tau,
                        status = 0)
  surv10 = rbind(surv10, surv_tmp) %>% arrange(id)
  
  surv_uni = rbind(surv00, surv10) %>% arrange(id)
  surv = rbind(surv_uni, surv_dup) %>% arrange(id)
  E = entry.time$entry.time[match(surv$id, entry.time$id)]
  surv$start = surv$start - E
  surv$stop = surv$stop - E
  
  return(surv)
}

get_curve_gen = function(dat, label){
  res = coxph(Surv(start, stop, status) ~ 1, data=dat)
  fit <- summary(survfit(res))
  dat_ret = data.frame(t = fit$time, Haz = -log(fit$surv), 
                       lower = -log(fit$upper), upper = -log(fit$lower), group = label)
  dat_tmp = data.frame(t = 0:750, Haz = rep(0, 751), group = label)
  dat_tmp$Haz = dat_ret$Haz[match(dat_tmp$t, dat_ret$t)]
  dat_tmp$Haz[1] = 0
  index_missing = which(is.na(dat_tmp$Haz))
  if(length(index_missing) > 0) {
    for (i in 1:length(index_missing)) {
      dat_tmp$Haz[index_missing[i]] = dat_tmp$Haz[index_missing[i] - 1]
    }
  }
  return(dat_tmp)
}

get_curve_svr = function(pred_svr, label, entry.time){
  dat = data.frame(id = pred_svr$id,
                   start = entry.time$entry.time,
                   stop = pred_svr$svr.infection.time,
                   status = ifelse(pred_svr$svr.infection.time < tau, 1, 0))
  E = entry.time$entry.time[match(dat$id, entry.time$id)]
  dat$start = dat$start - E
  dat$stop = dat$stop - E
  
  res = coxph(Surv(start, stop, status) ~ 1, data=dat)
  fit <- summary(survfit(res))
  dat_ret = data.frame(t = fit$time, Haz = -log(fit$surv), 
                       lower = -log(fit$upper), upper = -log(fit$lower), group = label)
  
  dat_tmp = data.frame(t = 0:750, Haz = rep(0, 751), group = label)
  dat_tmp$Haz = dat_ret$Haz[match(dat_tmp$t, dat_ret$t)]
  dat_tmp$Haz[1] = 0
  index_missing = which(is.na(dat_tmp$Haz))
  if(length(index_missing) > 0) {
    for (i in 1:length(index_missing)) {
      dat_tmp$Haz[index_missing[i]] = dat_tmp$Haz[index_missing[i] - 1]
    }
  }
  
  return(dat_tmp)
}

tau = 792
martb = 10
marlr = 10

dat_plot3 = data.frame(t = 0:750, Haz = rep(0, 751), group = "9 months")
dat_plot_svr3 = data.frame(t = 0:750, Haz = rep(0, 751), group = "9 months")
for (k in filename) {
  cat("k =", which(filename == k), "\n")
  load(file = paste("../output_age65+_3/", k, sep = ""))
  # General Infection
  dat = get_plot_data(pred3, entry.time.inf)
  dat3 = get_curve_gen(dat, "9 months")
  dat_plot3$Haz = dat_plot3$Haz + dat3$Haz
  
  # Severe Infection
  dat_svr3 = get_curve_svr(pred3_svr, "9 months", entry.time.svr)
  dat_plot_svr3$Haz = dat_plot_svr3$Haz + dat_svr3$Haz
}
dat_plot3$Haz = dat_plot3$Haz / length(filename)
dat_plot_svr3$Haz = dat_plot_svr3$Haz / length(filename)

save(dat_plot3, dat_plot_svr3, file = "plot_age65+_3.RData")




