rm(list = ls())
library(data.table)
library(dplyr)
library(Rcpp)
seed <- as.integer(abs(rnorm(1) * 100000))
set.seed(seed)
Sys.setenv("PKG_CXXFLAGS" = "-fopenmp")
Sys.setenv("PKG_LIBS" = "-fopenmp")
# setwd("/Users/jianchen/Library/CloudStorage/Dropbox/Yang/UNC/covid_optimal_booster/prediction/original/big_model_severe_infection_age65+")
# load("../tab_all_processed.RData")
# sourceCpp("../Cox_booster.cpp")
# sourceCpp("../computeList.cpp")

load("../tab_all_processed.RData")
sourceCpp("../Cox_booster_parallel.cpp")
sourceCpp("../computeList.cpp")

index_missing = which(is.na(tab_all_processed$death.time) & is.na(tab_all_processed$hosp.time))
tab_all_processed = tab_all_processed[-index_missing,]

tau = 792
cohort_cutoff1 = c(-Inf, 329, 479, 629, Inf)
cohort_cutoff2 = c(-Inf, 629, Inf)
cohort_cutoff3 = c(-Inf, 629, Inf)
age_cutoff = c(-Inf, 75, Inf)

survData_mixed = tab_all_processed %>%
  filter(age == ">=65") %>%
  mutate(age = cut(age_continuous, breaks = age_cutoff)) %>%
  mutate(booster1.cohort = cut(booster1.date, breaks = cohort_cutoff1), 
         booster2.cohort = cut(booster2.date, breaks = cohort_cutoff2), 
         booster3.cohort = cut(booster3.date, breaks = cohort_cutoff3), 
         event.time = ifelse(infection.time > entry.time & pmin(hosp.time, death.time, na.rm = TRUE) < tau, infection.time, tau),
         infection.time = ifelse(infection.time > entry.time & pmin(hosp.time, death.time, na.rm = TRUE) < tau, tau, infection.time),
         censor.time = pmin(tau, event.time), event.status = ifelse(event.time < tau, 1, 0)) %>%
  select(cluster_id, event.time, event.status, censor.time, 
         age, age_continuous, gender, GeoRegion, VacRegion, RaceEth, dep_status,
         infection.time, entry.period, entry.time, primary.type,
         booster1.date, booster1.manu, booster1.cohort, booster1.type,
         booster2.date, booster2.manu, booster2.cohort, booster2.type,
         booster3.date, booster3.manu, booster3.cohort, booster3.type)

n = length(unique(survData_mixed$cluster_id))
index_dup = which(duplicated(survData_mixed$cluster_id, fromLast = T))
survData_uni = survData_mixed[-index_dup,]
X = model.matrix(~ gender + age + GeoRegion + VacRegion + RaceEth + dep_status + entry.period + primary.type, survData_uni)[,-1]
Xg = matrix(0, nrow = 1, ncol = dim(X)[2])
ng = 0

survData_uni_tmp = survData_uni

comb1 = survData_uni %>%
  select(booster1.manu, booster1.cohort, booster1.type) %>%
  na.omit() %>%
  unique() %>%
  arrange(booster1.cohort)
comb1$type.num1 = c(1, 2, 3, 4)

comb2 = survData_uni %>%
  select(booster2.manu, booster2.cohort, booster2.type) %>%
  na.omit() %>%
  unique() %>%
  arrange(booster2.cohort)
comb2$type.num2 = c(5, 6)

comb3 = survData_uni %>%
  select(booster3.manu, booster3.cohort, booster3.type) %>%
  na.omit() %>%
  unique() %>%
  arrange(booster3.cohort)
comb3$type.num3 = c(5, 7)

booster.num  = length(unique(c(comb1$type.num1, comb2$type.num2, comb3$type.num3)))

survData_uni = merge(survData_uni, comb1, 
                     by = c("booster1.manu", "booster1.cohort", "booster1.type"), 
                     all = TRUE, sort = FALSE)
survData_uni = merge(survData_uni, comb2, 
                     by = c("booster2.manu", "booster2.cohort", "booster2.type"), 
                     all = TRUE, sort = FALSE)
survData_uni = merge(survData_uni, comb3, 
                     by = c("booster3.manu", "booster3.cohort", "booster3.type"), 
                     all = TRUE, sort = FALSE)

S_booster1 = ifelse(is.na(survData_uni$booster1.date), tau, survData_uni$booster1.date)
V_booster1 = ifelse(is.na(survData_uni$type.num1), comb1$type.num1[1], survData_uni$type.num1)
S_booster2 = ifelse(is.na(survData_uni$booster2.date), tau, survData_uni$booster2.date)
V_booster2 = ifelse(is.na(survData_uni$type.num2), comb2$type.num2[1], survData_uni$type.num2)
S_booster3 = ifelse(is.na(survData_uni$booster3.date), tau, survData_uni$booster3.date)
V_booster3 = ifelse(is.na(survData_uni$type.num3), comb3$type.num3[1], survData_uni$type.num3)

S = computeList(as.list(S_booster1), as.list(S_booster2), (1:n)-1)
S = computeList(S, as.list(S_booster3), (1:n)-1)
V = computeList(as.list(V_booster1), as.list(V_booster2), (1:n)-1)
V = computeList(V, as.list(V_booster3), (1:n)-1)
rm(S_booster1, V_booster1, S_booster2, V_booster2, S_booster3, V_booster3)

## Infection
start.date = as.Date("20201210", "%Y%m%d")
end.date = c(as.Date("20211215", "%Y%m%d"))
endpt = difftime(end.date, start.date, units = "days")
endpt = as.numeric(endpt)
nmos = length(endpt)+1
n_cohort = booster.num

m = table(survData_mixed$cluster_id)
S_inf1 = survData_mixed$infection.time[-which(duplicated(survData_mixed$cluster_id, fromLast = F))]
V_inf1 = as.numeric(cut(S_inf1, breaks = c(-Inf, endpt, Inf), labels = 1:nmos)) + n_cohort
S = computeList(S, as.list(S_inf1), (1:n)-1)
V = computeList(V, as.list(V_inf1), (1:n)-1)
rm(S_inf1, V_inf1)

index2 = as.numeric(attr(m, "dimnames")[[1]])[which(m >= 2)]
loc2 = match(index2, survData_uni$cluster_id)
dat2 = survData_mixed[which(survData_mixed$cluster_id %in% index2),]
dat2 = dat2[duplicated(dat2$cluster_id, fromLast = F),]
infection2 = dat2$infection.time[!duplicated(dat2$cluster_id, fromLast = F)]
S_inf2 = rep(tau+1, n)
S_inf2[loc2] = infection2
V_inf2 = as.numeric(cut(S_inf2, breaks = c(-Inf, endpt, Inf), labels = 1:nmos)) + n_cohort
S = computeList(S, as.list(S_inf2), (1:n)-1)
V = computeList(V, as.list(V_inf2), (1:n)-1)
rm(S_inf2, V_inf2)

index3 = as.numeric(attr(m, "dimnames")[[1]])[which(m >= 3)]
loc3 = match(index3, survData_uni$cluster_id)
dat3 = survData_mixed[which(survData_mixed$cluster_id %in% index3),]
dat3 = dat3[duplicated(dat3$cluster_id, fromLast = F),]
dat3 = dat3[duplicated(dat3$cluster_id, fromLast = F),]
infection3 = dat3$infection.time[!duplicated(dat3$cluster_id, fromLast = F)]
S_inf3 = rep(tau+1, n)
S_inf3[loc3] = infection3
V_inf3 = as.numeric(cut(S_inf3, breaks = c(-Inf, endpt, Inf), labels = 1:nmos)) + n_cohort
S = computeList(S, as.list(S_inf3), (1:n)-1)
V = computeList(V, as.list(V_inf3), (1:n)-1)
rm(S_inf3, V_inf3)

Time = survData_uni$event.time
Delta = survData_uni$event.status
t = sort(unique(Time[which(Delta==1)]), decreasing = T)
m = rep(1, n)
C = survData_uni$censor.time
VacTime = survData_uni$entry.time


knots = list("type1" = c(14),
             "type2" = c(14),
             "type3" = c(14),
             "type4" = c(14),
             "type5" = c(14),
             "type6" = c(14),
             "type7" = c(14),
             "infection1" = c(90),
             "infection2" = c(90))

infection_ind = 8
constantVE = FALSE
dimension = numeric(length(knots))
for (i in 1:length(dimension)) {
  dimension[i] = length(knots[[i]]) + 1
}
if(constantVE){
  dimension = dimension - 1
}

save(VacTime, file = "../entry_time_age65+_svr.RData")

survData_uni = survData_uni_tmp
save(tau, cohort_cutoff1, cohort_cutoff2, cohort_cutoff3, X, comb1, comb2, comb3, n, n_cohort,
     survData_uni, booster.num, endpt, knots, constantVE, dimension, infection_ind, nmos, 
     VacTime, file = "dat_severe_infection_age65+.RData")

result = Cox_general(Time, t, Delta, C, V, S, X, m, Xg, ng, eps = 1e-3, MaxIter = 10, 
                     knots, constantVE, dimension, infection_ind, VacTime, 90)
save(result, file = "res_big_model_severe_infection_age65+.RData")
